"""
KpatchPatches - report locally stored kpatch patches
====================================================

This parser creates a list of the module names of locally
stored kpatch modules returned by command ``ls /var/lib/kpatch/\`uname -r\`/``.
If no modules are installed, a ContentException will be raised.

"""

from .. import parser, CommandParser
from insights.specs import Specs


@parser(Specs.kpatch_patch_files)
class KpatchPatches(CommandParser):
    """
    A parser for getting modules names of locally stored kpatch-patch files.

    Sample output of `ls /var/lib/kpatch/\`uname -r\`/` looks like::

        kpatch-3_10_0-1062-1-5.ko kpatch-3_10_0-1062-1-6.ko

    Attributes:
        patches (str): List of the name of kpatch patches. The dashes are
            converted to underscores, file suffix are removed, and duplicated
            names are removed as well

    Examples:
        >>> kp.patches
        ['kpatch_3_10_0_1062_1_5', 'kpatch_3_10_0_1062_1_6']
    """

    def parse_content(self, content):
        # convert dashes to underscores, remove file suffixes, remove duplicates
        self.patches = list(set([p.split('.')[0].replace("-", "_") for p in content]))



"""
Utility functions
"""
from __future__ import absolute_import
import glob
import socket
import os
import logging
import uuid
import datetime
import shlex
import re
import sys
from subprocess import Popen, PIPE, STDOUT

import yaml
try:
    from yaml import CLoader as Loader, CDumper as Dumper
except ImportError:
    from yaml import Loader, Dumper

from .. import package_info
from .constants import InsightsConstants as constants
from .collection_rules import InsightsUploadConf

logger = logging.getLogger(__name__)


def determine_hostname(display_name=None):
    """
    Find fqdn if we can
    """
    if display_name:
        # if display_name is provided, just return the given name
        return display_name
    else:
        socket_gethostname = socket.gethostname()
        socket_fqdn = socket.getfqdn()

        try:
            socket_ex = socket.gethostbyname_ex(socket_gethostname)[0]
        except (LookupError, socket.gaierror):
            socket_ex = ''

        gethostname_len = len(socket_gethostname)
        fqdn_len = len(socket_fqdn)
        ex_len = len(socket_ex)

        if fqdn_len > gethostname_len or ex_len > gethostname_len:
            if "localhost" not in socket_ex and len(socket_ex):
                return socket_ex
            if "localhost" not in socket_fqdn:
                return socket_fqdn

        return socket_gethostname


def get_time():
    return datetime.datetime.isoformat(datetime.datetime.now())


def write_registered_file():
    delete_unregistered_file()
    for f in constants.registered_files:
        if os.path.lexists(f):
            if os.path.islink(f):
                # kill symlinks and regenerate
                os.remove(f)
                write_to_disk(f)
        else:
            write_to_disk(f)


def write_unregistered_file(date=None):
    """
    Write .unregistered out to disk
    """
    delete_registered_file()
    if date is None:
        date = get_time()
    for f in constants.unregistered_files:
        if os.path.lexists(f):
            if os.path.islink(f):
                # kill symlinks and regenerate
                os.remove(f)
                write_to_disk(f, content=str(date))
        else:
            write_to_disk(f, content=str(date))


def delete_registered_file():
    for f in constants.registered_files:
        write_to_disk(f, delete=True)


def delete_unregistered_file():
    for f in constants.unregistered_files:
        write_to_disk(f, delete=True)
    # this function only called when machine is registered,
    #  so while registering, delete this file too. we only
    #  need it around until we're registered
    write_to_disk(constants.register_marker_file, delete=True)


def delete_cache_files():
    for f in glob.glob(os.path.join(constants.insights_core_lib_dir, "*.json")):
        os.remove(f)


def write_to_disk(filename, delete=False, content=get_time()):
    """
    Write filename out to disk
    """
    if not os.path.exists(os.path.dirname(filename)):
        return
    if delete:
        if os.path.lexists(filename):
            os.remove(filename)
    else:
        with open(filename, 'wb') as f:
            f.write(content.encode('utf-8'))


def generate_machine_id(new=False,
                        destination_file=constants.machine_id_file):
    """
    Generate a machine-id if /etc/insights-client/machine-id does not exist
    """
    machine_id = None
    machine_id_file = None
    logging_name = 'machine-id'

    if os.path.isfile(destination_file) and not new:
        logger.debug('Found %s', destination_file)
        with open(destination_file, 'r') as machine_id_file:
            machine_id = machine_id_file.read()
    else:
        logger.debug('Could not find %s file, creating', logging_name)
        machine_id = str(uuid.uuid4())
        logger.debug("Creating %s", destination_file)
        write_to_disk(destination_file, content=machine_id)

    try:
        uuid.UUID(machine_id, version=4)
        return str(machine_id).strip()
    except ValueError:
        logger.error("Invalid machine ID: %s", machine_id)
        logger.error("Remove %s and a new one will be generated.\nRerun the client with --register", destination_file)
        sys.exit(constants.sig_kill_bad)


def _expand_paths(path):
    """
    Expand wildcarded paths
    """
    dir_name = os.path.dirname(path)
    paths = []
    logger.debug("Attempting to expand %s", path)
    if os.path.isdir(dir_name):
        files = os.listdir(dir_name)
        match = os.path.basename(path)
        for file_path in files:
            if re.match(match, file_path):
                expanded_path = os.path.join(dir_name, file_path)
                paths.append(expanded_path)
        logger.debug("Expanded paths %s", paths)
        return paths
    else:
        logger.debug("Could not expand %s", path)


def validate_remove_file(config):
    """
    Validate the remove file
    """
    return InsightsUploadConf(config).validate()


def write_data_to_file(data, filepath):
    '''
    Write data to file
    '''
    try:
        os.makedirs(os.path.dirname(filepath), 0o700)
    except OSError:
        pass

    write_to_disk(filepath, content=data)


def magic_plan_b(filename):
    '''
    Use this in instances where
    python-magic is MIA and can't be installed
    for whatever reason
    '''
    cmd = shlex.split('file --mime-type --mime-encoding ' + filename)
    stdout, stderr = Popen(cmd, stdout=PIPE).communicate()
    stdout = stdout.decode("utf-8")
    mime_str = stdout.split(filename + ': ')[1].strip()
    return mime_str


def run_command_get_output(cmd):
    proc = Popen(shlex.split(cmd),
                 stdout=PIPE, stderr=STDOUT)
    stdout, stderr = proc.communicate()

    return {
        'status': proc.returncode,
        'output': stdout.decode('utf-8', 'ignore')
    }


def modify_config_file(updates):
    '''
    Update the config file with certain things
    '''
    cmd = '/bin/sed '
    for key in updates:
        cmd = cmd + '-e \'s/^#*{key}.*=.*$/{key}={value}/\' '.format(key=key, value=updates[key])
    cmd = cmd + constants.default_conf_file
    status = run_command_get_output(cmd)
    write_to_disk(constants.default_conf_file, content=status['output'])


def get_version_info():
    '''
    Get the insights client and core versions for archival
    '''
    cmd = 'rpm -q --qf "%{VERSION}-%{RELEASE}" insights-client'
    version_info = {}
    version_info['core_version'] = '%s-%s' % (package_info['VERSION'], package_info['RELEASE'])
    rpm_proc = run_command_get_output(cmd)
    if rpm_proc['status'] != 0:
        # Unrecoverable error
        logger.debug('Error occurred while running rpm -q. Details:\n%s' % rpm_proc['output'])
        version_info['client_version'] = None
    else:
        version_info['client_version'] = rpm_proc['output']
    return version_info


def print_egg_versions():
    '''
    Log all available eggs' versions
    '''
    versions = get_version_info()
    logger.debug('Client version: %s', versions['client_version'])
    logger.debug('Core version: %s', versions['core_version'])
    logger.debug('All egg versions:')
    eggs = [
        os.getenv('EGG'),
        '/var/lib/insights/newest.egg',
        '/var/lib/insights/last_stable.egg',
        '/etc/insights-client/rpm.egg',
    ]
    if not sys.executable:
        logger.debug('Python executable not found.')
        return

    for egg in eggs:
        if egg is None:
            logger.debug('ENV egg not defined.')
            continue
        if not os.path.exists(egg):
            logger.debug('%s not found.', egg)
            continue
        try:
            proc = Popen([sys.executable, '-c',
                         'from insights import package_info; print(\'%s-%s\' % (package_info[\'VERSION\'], package_info[\'RELEASE\']))'],
                         env={'PYTHONPATH': egg, 'PATH': os.getenv('PATH')}, stdout=PIPE, stderr=STDOUT)
        except OSError:
            logger.debug('Could not start python.')
            return
        stdout, stderr = proc.communicate()
        version = stdout.decode('utf-8', 'ignore').strip()
        logger.debug('%s: %s', egg, version)


def read_pidfile():
    '''
    Read the pidfile we wrote at launch
    '''
    pid = None
    try:
        with open(constants.pidfile) as pidfile:
            pid = pidfile.read()
    except IOError:
        logger.debug('Could not open pidfile for reading.')
    return pid


def systemd_notify(pid):
    '''
    Ping the systemd watchdog with the main PID so that
    the watchdog doesn't kill the process
    '''
    if not os.getenv('NOTIFY_SOCKET'):
        # running standalone, not via systemd job
        return
    if not pid:
        logger.debug('No PID specified.')
        return
    if not os.path.exists('/usr/bin/systemd-notify'):
        # RHEL 6, no systemd
        return
    try:
        proc = Popen(['/usr/bin/systemd-notify', '--pid=' + str(pid), 'WATCHDOG=1'])
    except OSError:
        logger.debug('Could not launch systemd-notify.')
        return
    stdout, stderr = proc.communicate()
    if proc.returncode != 0:
        logger.debug('systemd-notify returned %s', proc.returncode)


def get_tags(tags_file_path=os.path.join(constants.default_conf_dir, "tags.conf")):
    '''
    Load tag data from the tags file.

    Returns: a dict containing tags defined on the host.
    '''
    tags = None

    try:
        with open(tags_file_path) as f:
            data = f.read()
            tags = yaml.load(data, Loader=Loader)
    except EnvironmentError as e:
        logger.debug("tags file does not exist: %s", os.strerror(e.errno))

    return tags


def write_tags(tags, tags_file_path=os.path.join(constants.default_conf_dir, "tags.conf")):
    """
    Writes tags to tags_file_path

    Arguments:
      - tags (dict): the tags to write
      - tags_file_path (string): path to which tag data will be written

    Returns: None
    """
    with open(tags_file_path, mode="w") as f:
        data = yaml.dump(tags, Dumper=Dumper, default_flow_style=False)
        f.write(data)



import logging
import base64
import json
from requests import ConnectionError, Timeout
from requests.exceptions import HTTPError, MissingSchema
from ssl import SSLError
from urllib3.exceptions import MaxRetryError
from insights.client.connection import InsightsConnection
from insights.client.schedule import get_scheduler
from insights.client.constants import InsightsConstants as constants
from insights.client.utilities import write_to_disk

logger = logging.getLogger(__name__)
net_logger = logging.getLogger('network')

IDENTITY_URI = 'http://169.254.169.254/latest/dynamic/instance-identity'
IDENTITY_DOC_URI = IDENTITY_URI + '/document'
IDENTITY_PKCS7_URI = IDENTITY_URI + '/pkcs7'


def aws_main(config):
    '''
    Process AWS entitlements with Hydra
    '''
    if config.authmethod != 'BASIC':
        logger.error('AWS entitlement is only available when BASIC auth is used.\n'
                     'Set auto_config=False and authmethod=BASIC in %s.', config.conf)
        return False
    # workaround for a workaround
    #   the hydra API doesn't accept the legacy cert
    #   and legacy_upload=False currently just
    #   redirects to the classic API with /platform added
    #   so if doing AWS entitlement, use cert_verify=True
    config.cert_verify = True
    conn = InsightsConnection(config)

    bundle = get_aws_identity(conn)
    if not bundle:
        return False
    succ = post_to_hydra(conn, bundle)
    if not succ:
        return False
    # register with insights if this option
    #   wasn't specified
    if not config.portal_access_no_insights:
        enable_delayed_registration(config)
    return True


def get_uri(conn, uri):
    '''
    Fetch information from URIs
    '''
    try:
        net_logger.info('GET %s', uri)
        res = conn.session.get(uri, timeout=conn.config.http_timeout)
    except (ConnectionError, Timeout) as e:
        logger.error(e)
        logger.error('Could not reach %s', uri)
        return None
    net_logger.info('Status code: %s', res.status_code)
    return res


def get_aws_identity(conn):
    '''
    Get data from AWS
    '''
    logger.info('Fetching AWS identity information.')
    doc_res = get_uri(conn, IDENTITY_DOC_URI)
    pkcs7_res = get_uri(conn, IDENTITY_PKCS7_URI)
    if not (doc_res and pkcs7_res) or not (doc_res.ok and pkcs7_res.ok):
        logger.error('Error getting identity information.')
        return None
    logger.debug('Identity information obtained successfully.')
    identity_doc = base64.b64encode(doc_res.content)

    return {
        'document': identity_doc.decode('utf-8'),
        'pkcs7': pkcs7_res.text
    }


def post_to_hydra(conn, data):
    '''
    Post data to Hydra
    '''
    logger.info('Submitting identity information to Red Hat.')
    hydra_endpoint = conn.config.portal_access_hydra_url

    # POST to hydra
    try:
        json_data = json.dumps(data)
        net_logger.info('POST %s', hydra_endpoint)
        net_logger.info('POST body: %s', json_data)
        res = conn.session.post(hydra_endpoint, data=json_data, timeout=conn.config.http_timeout)
    except MissingSchema as e:
        logger.error(e)
        return False
    except (ConnectionError, Timeout, SSLError, MaxRetryError) as e:
        logger.error(e)
        logger.error('Could not reach %s', hydra_endpoint)
        return False
    net_logger.info('Status code: %s', res.status_code)
    try:
        res.raise_for_status()
    except HTTPError as e:
        # if failure,
        # error, return False
        logger.error(e)
        try:
            res_json = res.json()
            err_msg = res_json.get('message', '')
            err_details = res_json.get('detailMessage', '')
            logger.error('%s\n%s', err_msg, err_details)
        except ValueError:
            logger.error('Could not parse JSON response.')
        return False
    logger.info('Entitlement information has been sent.')
    return True


def enable_delayed_registration(config):
    '''
    Write a marker file to allow client to know that
    it should attempt to register when it runs
    '''
    logger.debug('Writing to %s', constants.register_marker_file)
    write_to_disk(constants.register_marker_file)
    job = get_scheduler(config)
    job.set_daily()



import logging
import os

from insights.core import archives
from insights.core.context import (ClusterArchiveContext,
                                   ExecutionContextMeta,
                                   HostArchiveContext)

log = logging.getLogger(__name__)


def get_all_files(path):
    all_files = []
    for f in archives.get_all_files(path):
        if os.path.isfile(f) and not os.path.islink(f):
            all_files.append(f)
    return all_files


def identify(files):
    common_path, ctx = ExecutionContextMeta.identify(files)
    if ctx:
        return common_path, ctx

    common_path = os.path.dirname(os.path.commonprefix(files))
    if not common_path:
        raise archives.InvalidArchive("Unable to determine common path")

    return common_path, HostArchiveContext


def create_context(path, context=None):
    top = os.listdir(path)
    arc = [os.path.join(path, f) for f in top if f.endswith(archives.COMPRESSION_TYPES)]
    if arc:
        return ClusterArchiveContext(path, all_files=arc)

    all_files = get_all_files(path)
    if not all_files:
        raise archives.InvalidArchive("No files in archive")

    common_path, ctx = identify(all_files)
    context = context or ctx
    return context(common_path, all_files=all_files)



"""
Rules for data collection
"""
from __future__ import absolute_import
import hashlib
import json
import logging
import six
import shlex
import os
import requests
import yaml
import stat
from six.moves import configparser as ConfigParser

from subprocess import Popen, PIPE, STDOUT
from tempfile import NamedTemporaryFile
from .constants import InsightsConstants as constants

APP_NAME = constants.app_name
logger = logging.getLogger(__name__)
net_logger = logging.getLogger('network')

expected_keys = ('commands', 'files', 'patterns', 'keywords')


class InsightsUploadConf(object):
    """
    Insights spec configuration from uploader.json
    """

    def __init__(self, config, conn=None):
        """
        Load config from parent
        """
        self.config = config
        self.fallback_file = constants.collection_fallback_file
        self.remove_file = config.remove_file
        self.collection_rules_file = constants.collection_rules_file
        self.collection_rules_url = self.config.collection_rules_url
        self.gpg = self.config.gpg
        if conn:
            if self.collection_rules_url is None:
                if config.legacy_upload:
                    self.collection_rules_url = conn.base_url + '/v1/static/uploader.v2.json'
                else:
                    self.collection_rules_url = conn.base_url.split('/platform')[0] + '/v1/static/uploader.v2.json'
                    # self.collection_rules_url = conn.base_url + '/static/uploader.v2.json'
            self.conn = conn

    def validate_gpg_sig(self, path, sig=None):
        """
        Validate the collection rules
        """
        logger.debug("Verifying GPG signature of Insights configuration")
        if sig is None:
            sig = path + ".asc"
        command = ("/usr/bin/gpg --no-default-keyring "
                   "--keyring " + constants.pub_gpg_path +
                   " --verify " + sig + " " + path)
        if not six.PY3:
            command = command.encode('utf-8', 'ignore')
        args = shlex.split(command)
        logger.debug("Executing: %s", args)
        proc = Popen(
            args, shell=False, stdout=PIPE, stderr=STDOUT, close_fds=True)
        stdout, stderr = proc.communicate()
        logger.debug("STDOUT: %s", stdout)
        logger.debug("STDERR: %s", stderr)
        logger.debug("Status: %s", proc.returncode)
        if proc.returncode:
            logger.error("ERROR: Unable to validate GPG signature: %s", path)
            return False
        else:
            logger.debug("GPG signature verified")
            return True

    def try_disk(self, path, gpg=True):
        """
        Try to load json off disk
        """
        if not os.path.isfile(path):
            return

        if not gpg or self.validate_gpg_sig(path):
            stream = open(path, 'r')
            json_stream = stream.read()
            if len(json_stream):
                try:
                    json_config = json.loads(json_stream)
                    return json_config
                except ValueError:
                    logger.error("ERROR: Invalid JSON in %s", path)
                    return False
            else:
                logger.warn("WARNING: %s was an empty file", path)
                return

    def get_collection_rules(self, raw=False):
        """
        Download the collection rules
        """
        logger.debug("Attemping to download collection rules from %s",
                     self.collection_rules_url)

        net_logger.info("GET %s", self.collection_rules_url)
        try:
            req = self.conn.session.get(
                self.collection_rules_url, headers=({'accept': 'text/plain'}))

            if req.status_code == 200:
                logger.debug("Successfully downloaded collection rules")

                json_response = NamedTemporaryFile()
                json_response.write(req.text.encode('utf-8'))
                json_response.file.flush()
            else:
                logger.error("ERROR: Could not download dynamic configuration")
                logger.error("Debug Info: \nConf status: %s", req.status_code)
                logger.error("Debug Info: \nConf message: %s", req.text)
                return None
        except requests.ConnectionError as e:
            logger.error(
                "ERROR: Could not download dynamic configuration: %s", e)
            return None

        if self.gpg:
            self.get_collection_rules_gpg(json_response)

        self.write_collection_data(self.collection_rules_file, req.text)

        if raw:
            return req.text
        else:
            return json.loads(req.text)

    def fetch_gpg(self):
        logger.debug("Attemping to download collection "
                     "rules GPG signature from %s",
                     self.collection_rules_url + ".asc")

        headers = ({'accept': 'text/plain'})
        net_logger.info("GET %s", self.collection_rules_url + '.asc')
        config_sig = self.conn.session.get(self.collection_rules_url + '.asc',
                                           headers=headers)
        if config_sig.status_code == 200:
            logger.debug("Successfully downloaded GPG signature")
            return config_sig.text
        else:
            logger.error("ERROR: Download of GPG Signature failed!")
            logger.error("Sig status: %s", config_sig.status_code)
            return False

    def get_collection_rules_gpg(self, collection_rules):
        """
        Download the collection rules gpg signature
        """
        sig_text = self.fetch_gpg()
        sig_response = NamedTemporaryFile(suffix=".asc")
        sig_response.write(sig_text.encode('utf-8'))
        sig_response.file.flush()
        self.validate_gpg_sig(collection_rules.name, sig_response.name)
        self.write_collection_data(self.collection_rules_file + ".asc", sig_text)

    def write_collection_data(self, path, data):
        """
        Write collections rules to disk
        """
        flags = os.O_WRONLY | os.O_CREAT | os.O_TRUNC
        fd = os.open(path, flags, 0o600)
        with os.fdopen(fd, 'w') as dyn_conf_file:
            dyn_conf_file.write(data)

    def get_conf_file(self):
        """
        Get config from local config file, first try cache, then fallback.
        """
        for conf_file in [self.collection_rules_file, self.fallback_file]:
            logger.debug("trying to read conf from: " + conf_file)
            conf = self.try_disk(conf_file, self.gpg)

            if not conf:
                continue

            version = conf.get('version', None)
            if version is None:
                raise ValueError("ERROR: Could not find version in json")

            conf['file'] = conf_file
            logger.debug("Success reading config")
            logger.debug(json.dumps(conf))
            return conf

        raise ValueError("ERROR: Unable to download conf or read it from disk!")

    def get_conf_update(self):
        """
        Get updated config from URL, fallback to local file if download fails.
        """
        dyn_conf = self.get_collection_rules()

        if not dyn_conf:
            return self.get_conf_file()

        version = dyn_conf.get('version', None)
        if version is None:
            raise ValueError("ERROR: Could not find version in json")

        dyn_conf['file'] = self.collection_rules_file
        logger.debug("Success reading config")
        config_hash = hashlib.sha1(json.dumps(dyn_conf).encode('utf-8')).hexdigest()
        logger.debug('sha1 of config: %s', config_hash)
        return dyn_conf

    def get_rm_conf_old(self):
        """
        Get excluded files config from remove_file.
        """
        # Convert config object into dict
        logger.debug('Trying to parse as INI file.')
        parsedconfig = ConfigParser.RawConfigParser()

        try:
            parsedconfig.read(self.remove_file)
            rm_conf = {}
            for item, value in parsedconfig.items('remove'):
                if item not in expected_keys:
                    raise RuntimeError('Unknown section in remove.conf: ' + item +
                                       '\nValid sections are ' + ', '.join(expected_keys) + '.')
                if six.PY3:
                    rm_conf[item] = value.strip().encode('utf-8').decode('unicode-escape').split(',')
                else:
                    rm_conf[item] = value.strip().decode('string-escape').split(',')
            return rm_conf
        except ConfigParser.Error as e:
            # can't parse config file at all
            logger.debug(e)
            raise RuntimeError('ERROR: Cannot parse the remove.conf file as a YAML file '
                               'nor as an INI file. Please check the file formatting.\n'
                               'See %s for more information.' % self.config.logging_file)

    def get_rm_conf(self):
        '''
        Load remove conf. If it's a YAML-formatted file, try to load
        the "new" version of remove.conf
        '''
        def is_list_of_strings(data):
            '''
            Helper function for correct_format()
            '''
            if data is None:
                # nonetype, no data to parse. treat as empty list
                return True
            if not isinstance(data, list):
                return False
            for l in data:
                if not isinstance(l, six.string_types):
                    return False
            return True

        def correct_format(parsed_data):
            '''
            Ensure the parsed file matches the needed format
            Returns True, <message> on error
            '''
            # validate keys are what we expect
            keys = parsed_data.keys()
            invalid_keys = set(keys).difference(expected_keys)
            if invalid_keys:
                return True, ('Unknown section(s) in remove.conf: ' + ', '.join(invalid_keys) +
                              '\nValid sections are ' + ', '.join(expected_keys) + '.')

            # validate format (lists of strings)
            for k in expected_keys:
                if k in parsed_data:
                    if k == 'patterns' and isinstance(parsed_data['patterns'], dict):
                        if 'regex' not in parsed_data['patterns']:
                            return True, 'Patterns section contains an object but the "regex" key was not specified.'
                        if 'regex' in parsed_data['patterns'] and len(parsed_data['patterns']) > 1:
                            return True, 'Unknown keys in the patterns section. Only "regex" is valid.'
                        if not is_list_of_strings(parsed_data['patterns']['regex']):
                            return True, 'regex section under patterns must be a list of strings.'
                        continue
                    if not is_list_of_strings(parsed_data[k]):
                        return True, '%s section must be a list of strings.' % k
            return False, None

        if not os.path.isfile(self.remove_file):
            logger.debug('No remove.conf defined. No files/commands will be ignored.')
            return None
        try:
            with open(self.remove_file) as f:
                rm_conf = yaml.safe_load(f)
            if rm_conf is None:
                logger.warn('WARNING: Remove file %s is empty.', self.remove_file)
                return {}
        except (yaml.YAMLError, yaml.parser.ParserError) as e:
            # can't parse yaml from conf, try old style
            logger.debug('ERROR: Cannot parse remove.conf as a YAML file.\n'
                         'If using any YAML tokens such as [] in an expression, '
                         'be sure to wrap the expression in quotation marks.\n\nError details:\n%s\n', e)
            return self.get_rm_conf_old()
        if not isinstance(rm_conf, dict):
            # loaded data should be a dict with at least one key (commands, files, patterns, keywords)
            logger.debug('ERROR: Invalid YAML loaded.')
            return self.get_rm_conf_old()
        err, msg = correct_format(rm_conf)
        if err:
            # YAML is correct but doesn't match the format we need
            raise RuntimeError('ERROR: ' + msg)
        # remove Nones, empty strings, and empty lists
        filtered_rm_conf = dict((k, v) for k, v in rm_conf.items() if v)
        return filtered_rm_conf

    def validate(self):
        '''
        Validate remove.conf
        '''
        if not os.path.isfile(self.remove_file):
            logger.warn("WARNING: Remove file does not exist")
            return False
        # Make sure permissions are 600
        mode = stat.S_IMODE(os.stat(self.remove_file).st_mode)
        if not mode == 0o600:
            logger.error("ERROR: Invalid remove file permissions. "
                         "Expected 0600 got %s" % oct(mode))
            return False
        else:
            logger.debug("Correct file permissions")
        success = self.get_rm_conf()
        if success is None or success is False:
            logger.error('Could not parse remove.conf')
            return False
        # Using print here as this could contain sensitive information
        print('Remove file parsed contents:')
        print(success)
        logger.info('Parsed successfully.')
        return True


if __name__ == '__main__':
    from .config import InsightsConfig
    print(InsightsUploadConf(InsightsConfig().load_all()))



from insights.parsers import kpatch_patches
from insights.tests import context_wrap
from insights.core.plugins import ContentException
import pytest


ASSORTED_KPATCHES = """
asdfasdfasdf_asdfasdfasdf-asdfasdfasdf_asdfasdfasdf.ko
asdfasdfasdf_asdfasdfasdf-asdfasdfasdf_asdfasdfasdf.ko.xz
foo-bar.ko
foo-bar.ko.xz
foo.ko
foo.ko.xz
test_klp_callbacks_demo.ko
test_klp_callbacks_demo.ko.xz
""".strip()

NO_KPATCH = """
/bin/ls: cannot access '/var/lib/kpatch/4.18.0-147.8.el8.x86_64': No such file or directory
""".strip()


# Try a bunch of random potential patch names
# Compare to expected module names
def test_assorted():
    kp = kpatch_patches.KpatchPatches(context_wrap(ASSORTED_KPATCHES))
    for patch in [
            'asdfasdfasdf_asdfasdfasdf_asdfasdfasdf_asdfasdfasdf',
            'foo_bar',
            'foo',
            'test_klp_callbacks_demo']:
        assert patch in kp.patches


# Try the case of no patches installed
def test_no_kpatch():
    with pytest.raises(ContentException):
        kpatch_patches.KpatchPatches(context_wrap(NO_KPATCH))



"""
Top level OpenShift 4 component
===============================
The :py:func:`conf` component recognizes insights-operator and must-gather
archives.
"""
import logging
import os
import yaml

from fnmatch import fnmatch
from insights.core.plugins import component
from insights.core.context import InsightsOperatorContext, MustGatherContext

from insights.core.archives import extract
from insights.parsr.query import from_dict, Result
from insights.util import content_type


log = logging.getLogger(__name__)

try:
    # requires pyyaml installed after libyaml
    Loader = yaml.CSafeLoader
except:
    log.info("Couldn't find libyaml loader. Falling back to python loader.")
    Loader = yaml.SafeLoader


def _get_files(path):
    for root, dirs, names in os.walk(path):
        for name in names:
            yield os.path.join(root, name)


def _load(path):
    with open(path) as f:
        doc = yaml.load(f, Loader=Loader)
        return from_dict(doc)


def _process(path, excludes=None):
    excludes = excludes if excludes is not None else []
    for f in _get_files(path):
        if excludes and any(fnmatch(f, e) for e in excludes):
            continue
        try:
            yield _load(f)
        except Exception:
            log.debug("Failed to load %s; skipping.", f)


def analyze(paths, excludes=None):
    if not isinstance(paths, list):
        paths = [paths]

    results = []
    for path in paths:
        if content_type.from_file(path) == "text/plain":
            results.append(_load(path))
        elif os.path.isdir(path):
            results.extend(_process(path, excludes))
        else:
            with extract(path) as ex:
                results.extend(_process(ex.tmp_dir, excludes))

    return Result(children=results)


@component([InsightsOperatorContext, MustGatherContext])
def conf(io, mg):
    """
    The ``conf`` component parses all configuration in an insights-operator or
    must-gather archive and returns an object that is part of the parsr common
    data model.  It can be navigated and queried in a standard way. See the
    `tutorial`_ for details.

    .. _tutorial: https://insights-core.readthedocs.io/en/latest/notebooks/Parsr%20Query%20Tutorial.html
    """
    return analyze((io or mg).root)
